==> pan-0.13.3.tar.bz2: pan-0.13.3/pan/article-filter-ui.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <glib.h>
#include <gtk/gtk.h>

#include <pan/base/article.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>

#include <pan/filters/filter.h>
#include <pan/filters/filter-aggregate.h>
#include <pan/filters/filter-phrase.h>
#include <pan/filters/filter-top.h>
#include <pan/filters/filter-manager.h>

#include <pan/article-filter-ui.h>
#include <pan/globals.h>
#include <pan/util.h>

typedef struct
{
	GtkWidget * dialog;

	GtkWidget * bozo_author_tb;
	GtkWidget * bozo_author_entry;
	GtkWidget * bozo_author_regexp_tb;

	GtkWidget * spam_author_tb;
	GtkWidget * spam_author_entry;
	GtkWidget * spam_author_regexp_tb;

	GtkWidget * spam_subject_tb;
	GtkWidget * spam_subject_entry;
	GtkWidget * spam_subject_regexp_tb;
}
ArticleFilterDialog;



static void
add_to_filters (ArticleFilterDialog * d)
{
	gint i;
	Filter * spam;
	Filter * bozo;
	GPtrArray * filters;
	gboolean bozoize_author = FALSE;
	gboolean spamize_author = FALSE;
	gboolean spamize_subject = FALSE;

	/* get a copy of the filters, and find Spam and Bozo */	
	bozo = NULL;
	spam = NULL;
	filters = g_ptr_array_new ();
	filter_manager_get_filters (filters);
	for (i=0; i<filters->len; ++i) {
		FilterTop * f = FILTER_TOP(g_ptr_array_index(filters,i));
		if (!pan_strcmp (f->name, bozo_filter_name)) bozo = FILTER(f);
		if (!pan_strcmp (f->name, spam_filter_name)) spam = FILTER(f);
	}

	/* will we be making any changes? */
	bozoize_author = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->bozo_author_tb));
	spamize_author = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->spam_author_tb));
	spamize_subject = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->spam_subject_tb));

	/* ensure there's a bozo filter if we need one */
	if (bozo==NULL && bozoize_author) {
		bozo = filter_top_new ();
		filter_top_set_visible (FILTER_TOP(bozo), TRUE);
		filter_top_set_name (FILTER_TOP(bozo), bozo_filter_name);
		filter_aggregate_set_type (FILTER_AGGREGATE(bozo), AGGREGATE_TYPE_OR);
		g_ptr_array_add (filters, bozo);
	}

	/* ensure there's a spam filter if we need one */
	if (spam==NULL && (spamize_author || spamize_subject)) {
		spam = filter_top_new ();
		filter_top_set_visible (FILTER_TOP(spam), TRUE);
		filter_top_set_name (FILTER_TOP(spam), spam_filter_name);
		filter_aggregate_set_type (FILTER_AGGREGATE(spam), AGGREGATE_TYPE_OR);
		g_ptr_array_add (filters, spam);
	}

	/* update the bozo filter if necessary */
	if (bozoize_author)
	{
		Filter * filter = filter_phrase_new ();
		gchar * str = gtk_editable_get_chars (GTK_EDITABLE(d->bozo_author_entry), 0, -1);
		gboolean is_regexp = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->bozo_author_regexp_tb));
		g_strstrip (str);
		filter_phrase_set (FILTER_PHRASE(filter), FILTER_PHRASE_AUTHOR, str, is_regexp);
		filter_aggregate_add (FILTER_AGGREGATE(bozo), &filter, 1);
		g_free (str);
		pan_object_unref (PAN_OBJECT(filter));
	}

	/* update the spam filter if necessary */
	if (spamize_author)
	{
		Filter * filter = filter_phrase_new ();
		gchar * str = gtk_editable_get_chars (GTK_EDITABLE(d->spam_author_entry), 0, -1);
		gboolean is_regexp = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->spam_author_regexp_tb));
		g_strstrip (str);
		filter_phrase_set (FILTER_PHRASE(filter), FILTER_PHRASE_AUTHOR, str, is_regexp);
		filter_aggregate_add (FILTER_AGGREGATE(spam), &filter, 1);
		g_free (str);
		pan_object_unref (PAN_OBJECT(filter));
	}

	/* update the spam filter if necessary */
	if (spamize_subject)
	{
		Filter * filter = filter_phrase_new ();
		gchar * str = gtk_editable_get_chars (GTK_EDITABLE(d->spam_subject_entry), 0, -1);
		gboolean is_regexp = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->spam_subject_regexp_tb));
		g_strstrip (str);
		filter_phrase_set (FILTER_PHRASE(filter), FILTER_PHRASE_SUBJECT, str, is_regexp);
		filter_aggregate_add (FILTER_AGGREGATE(spam), &filter, 1);
		g_free (str);
		pan_object_unref (PAN_OBJECT(filter));
	}

	/* if we've made a change, update the filter manager */
	if (bozoize_author || spamize_author || spamize_subject)
		filter_manager_set_filters (filters);

	/* cleanup */
	pan_g_ptr_array_foreach (filters, (GFunc)pan_object_unref, NULL);
}


static gint
dialog_close_cb (GtkDialog * dialog, gpointer data)
{
	g_free (data);
	return FALSE;
}

static void
dialog_response_cb (GtkDialog * dialog, gint response, gpointer data)
{
	ArticleFilterDialog * d = (ArticleFilterDialog*) data;

	if (response==GTK_RESPONSE_ACCEPT)
	{
		add_to_filters (d);
	}
	if (response==GTK_RESPONSE_HELP)
	{
		GtkWidget * w = gtk_message_dialog_new (GTK_WINDOW(dialog), GTK_DIALOG_DESTROY_WITH_PARENT,
				 GTK_MESSAGE_INFO,
				 GTK_BUTTONS_CLOSE,
				 "%s", 
		_("Addresses and Subjects can be changed before being added\n"
"to the Spam/Bozo filters.\n"
"\n"
"For example, a subject of `-->MAKE MONEY FAST<--' will\n"
"likely find more spam if shortened to just `MAKE MONEY FAST'.\n"
"\n"
"Likewise, \"Dudley Dursley\" <ddursley@spambot.com>' could\n"
"be shortened to `<ddurseley@spambot.com>' or even `<*@spambot.com>'.\n"
"\n"
"Wildcards are supported and comparisons are case-insensitive.\n"
"For more complex pattern matching, enable the regular expression\n"
"button.  http://pan.rebelbase.com/regexp.txt for a tutorial.\n"));
	        gtk_widget_show_all (w);
		g_signal_connect_swapped (GTK_OBJECT(w), "response", 
		                          G_CALLBACK (gtk_widget_destroy), GTK_OBJECT(w));
	}
	if (response==GTK_RESPONSE_ACCEPT || response==GTK_RESPONSE_CLOSE)
	{
		gtk_widget_destroy (GTK_WIDGET(dialog));
	}
}

static void
activate_me_cb (GtkWidget * w, gpointer user_data)
{
	GtkToggleButton * tb = GTK_TOGGLE_BUTTON(user_data);
	gtk_toggle_button_set_active (tb, TRUE);
}

GtkWidget*
article_filter_dialog_new (GtkWidget * window, const Article * a)
{
	int row = 0;
	GtkWidget * w;
	GtkWidget * t;
	GtkWidget * tb;
	ArticleFilterDialog * d;
	char author[256];
	char * subject;
	const int atch = GTK_EXPAND | GTK_SHRINK | GTK_FILL;
	const int wide_enough = 250;

	/* filter criteria */
	article_get_author_str (a, author, sizeof(author));
	subject = g_strdup (article_get_subject(a));
	g_strstrip (subject);

	/* dialog */
	d = g_new0 (ArticleFilterDialog, 1);
	w = d->dialog = gtk_dialog_new_with_buttons (_("Pan: Filter Out Articles"),
	                                             GTK_WINDOW(window),
	                                             GTK_DIALOG_DESTROY_WITH_PARENT,
	                                             GTK_STOCK_HELP, GTK_RESPONSE_HELP,
	                                             GTK_STOCK_CANCEL, GTK_RESPONSE_CLOSE,
	                                             GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
	                                             NULL);
	gtk_window_set_policy (GTK_WINDOW(w), TRUE, TRUE, TRUE);
	g_signal_connect (w, "close", G_CALLBACK(dialog_close_cb), d);
	g_signal_connect (w, "response", G_CALLBACK(dialog_response_cb), d);

	/* main table */
	t = gtk_table_new (3, 3, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(t), GUI_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(t), GUI_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER(t), GUI_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(w)->vbox), t, TRUE, TRUE, 0);

	/* bozo author */
	d->bozo_author_tb = tb = w = gtk_check_button_new_with_label (_("Add Author to Bozo Filter"));
	gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+1, atch, atch, 0, 0);
	w = gtk_entry_new ();
	gtk_widget_set_usize (w, wide_enough, -1);
	d->bozo_author_entry = w;
	gtk_entry_set_text (GTK_ENTRY(w), author);
	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
	w = gtk_toggle_button_new_with_label (_("Regular Expression"));
	d->bozo_author_regexp_tb = w;
	g_signal_connect (w, "clicked", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+1, atch, atch, 0, 0);
	++row;

	/* spam author */
	d->spam_author_tb = tb = w = gtk_check_button_new_with_label (_("Add Author to Spam Filter"));
	gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+1, atch, atch, 0, 0);
	w = gtk_entry_new ();
	gtk_widget_set_usize (w, wide_enough, -1);
	d->spam_author_entry = w;
	gtk_entry_set_text (GTK_ENTRY(w), author);
	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
	w = gtk_toggle_button_new_with_label (_("Regular Expression"));
	d->spam_author_regexp_tb = w;
	g_signal_connect (w, "clicked", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+1, atch, atch, 0, 0);
	++row;

	/* spam subject */
	d->spam_subject_tb = w = tb = gtk_check_button_new_with_label (_("Add Subject to Spam Filter"));
	gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+1, atch, atch, 0, 0);
	w = gtk_entry_new ();
	gtk_widget_set_usize (w, wide_enough, -1);
	d->spam_subject_entry = w;
	gtk_entry_set_text (GTK_ENTRY(w), subject);
	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
	w = gtk_toggle_button_new_with_label (_("Regular Expression"));
	d->spam_subject_regexp_tb = w;
	g_signal_connect (w, "clicked", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+1, atch, atch, 0, 0);

	/* cleanup */
	g_free (subject);

	return d->dialog;
}

==> pan-0.13.4.tar.bz2: pan-0.13.4/pan/article-filter-ui.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <glib.h>
#include <gtk/gtk.h>

#include <pan/base/article.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>

#include <pan/filters/filter.h>
#include <pan/filters/filter-aggregate.h>
#include <pan/filters/filter-phrase.h>
#include <pan/filters/filter-top.h>
#include <pan/filters/filter-manager.h>

#include <pan/article-filter-ui.h>
#include <pan/globals.h>
#include <pan/util.h>

typedef struct
{
	GtkWidget * dialog;

	GtkWidget * bozo_author_tb;
	GtkWidget * bozo_author_entry;

	GtkWidget * spam_author_tb;
	GtkWidget * spam_author_entry;

	GtkWidget * spam_subject_tb;
	GtkWidget * spam_subject_entry;
}
ArticleFilterDialog;



static void
add_to_filters (ArticleFilterDialog * d)
{
	gint i;
	Filter * spam;
	Filter * bozo;
	GPtrArray * filters;
	gboolean bozoize_author = FALSE;
	gboolean spamize_author = FALSE;
	gboolean spamize_subject = FALSE;

	/* get a copy of the filters, and find Spam and Bozo */	
	bozo = NULL;
	spam = NULL;
	filters = g_ptr_array_new ();
	filter_manager_get_filters (filters);
	for (i=0; i<filters->len; ++i) {
		FilterTop * f = FILTER_TOP(g_ptr_array_index(filters,i));
		if (!pan_strcmp (f->name, bozo_filter_name)) bozo = FILTER(f);
		if (!pan_strcmp (f->name, spam_filter_name)) spam = FILTER(f);
	}

	/* will we be making any changes? */
	bozoize_author = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->bozo_author_tb));
	spamize_author = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->spam_author_tb));
	spamize_subject = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->spam_subject_tb));

	/* ensure there's a bozo filter if we need one */
	if (bozo==NULL && bozoize_author) {
		bozo = filter_top_new ();
		filter_top_set_visible (FILTER_TOP(bozo), TRUE);
		filter_top_set_name (FILTER_TOP(bozo), bozo_filter_name);
		filter_aggregate_set_type (FILTER_AGGREGATE(bozo), AGGREGATE_TYPE_OR);
		g_ptr_array_add (filters, bozo);
	}

	/* ensure there's a spam filter if we need one */
	if (spam==NULL && (spamize_author || spamize_subject)) {
		spam = filter_top_new ();
		filter_top_set_visible (FILTER_TOP(spam), TRUE);
		filter_top_set_name (FILTER_TOP(spam), spam_filter_name);
		filter_aggregate_set_type (FILTER_AGGREGATE(spam), AGGREGATE_TYPE_OR);
		g_ptr_array_add (filters, spam);
	}

	/* update the bozo filter if necessary */
	if (bozoize_author)
	{
		Filter * filter = filter_phrase_new ();
		char * str = gtk_editable_get_chars (GTK_EDITABLE(d->bozo_author_entry), 0, -1);
		g_strstrip (str);
		filter_phrase_set (FILTER_PHRASE(filter), PHRASE_MATCH_CONTAINS, PHRASE_KEY_AUTHOR, str);
		filter_aggregate_add (FILTER_AGGREGATE(bozo), &filter, 1);
		g_free (str);
		pan_object_unref (PAN_OBJECT(filter));
	}

	/* update the spam filter if necessary */
	if (spamize_author)
	{
		Filter * filter = filter_phrase_new ();
		char * str = gtk_editable_get_chars (GTK_EDITABLE(d->spam_author_entry), 0, -1);
		g_strstrip (str);
		filter_phrase_set (FILTER_PHRASE(filter), PHRASE_MATCH_CONTAINS, PHRASE_KEY_AUTHOR, str);
		filter_aggregate_add (FILTER_AGGREGATE(spam), &filter, 1);
		g_free (str);
		pan_object_unref (PAN_OBJECT(filter));
	}

	/* update the spam filter if necessary */
	if (spamize_subject)
	{
		Filter * filter = filter_phrase_new ();
		char * str = gtk_editable_get_chars (GTK_EDITABLE(d->spam_subject_entry), 0, -1);
		g_strstrip (str);
		filter_phrase_set (FILTER_PHRASE(filter), PHRASE_MATCH_CONTAINS, PHRASE_KEY_SUBJECT, str);
		filter_aggregate_add (FILTER_AGGREGATE(spam), &filter, 1);
		g_free (str);
		pan_object_unref (PAN_OBJECT(filter));
	}

	/* if we've made a change, update the filter manager */
	if (bozoize_author || spamize_author || spamize_subject)
		filter_manager_set_filters (filters);

	/* cleanup */
	pan_g_ptr_array_foreach (filters, (GFunc)pan_object_unref, NULL);
}


static gint
dialog_close_cb (GtkDialog * dialog, gpointer data)
{
	g_free (data);
	return FALSE;
}

static void
dialog_response_cb (GtkDialog * dialog, gint response, gpointer data)
{
	ArticleFilterDialog * d = (ArticleFilterDialog*) data;

	if (response==GTK_RESPONSE_ACCEPT)
	{
		add_to_filters (d);
	}
	if (response==GTK_RESPONSE_ACCEPT || response==GTK_RESPONSE_CLOSE)
	{
		gtk_widget_destroy (GTK_WIDGET(dialog));
	}
}

static void
activate_me_cb (GtkWidget * w, gpointer user_data)
{
	GtkToggleButton * tb = GTK_TOGGLE_BUTTON(user_data);
	gtk_toggle_button_set_active (tb, TRUE);
}

GtkWidget*
article_filter_dialog_new (GtkWidget * window, const Article * a)
{
	int row = 0;
	GtkWidget * w;
	GtkWidget * t;
	GtkWidget * tb;
	ArticleFilterDialog * d;
	char author[256];
	char * subject;
	const int atch = GTK_EXPAND | GTK_SHRINK | GTK_FILL;
	const int wide_enough = 250;

	/* filter criteria */
	article_get_author_str (a, author, sizeof(author));
	subject = g_strdup (article_get_subject(a));
	g_strstrip (subject);

	/* dialog */
	d = g_new0 (ArticleFilterDialog, 1);
	w = d->dialog = gtk_dialog_new_with_buttons (_("Pan: Filter Out Articles"),
	                                             GTK_WINDOW(window),
	                                             GTK_DIALOG_DESTROY_WITH_PARENT,
	                                             GTK_STOCK_CANCEL, GTK_RESPONSE_CLOSE,
	                                             GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
	                                             NULL);
	gtk_window_set_policy (GTK_WINDOW(w), TRUE, TRUE, TRUE);
	g_signal_connect (w, "close", G_CALLBACK(dialog_close_cb), d);
	g_signal_connect (w, "response", G_CALLBACK(dialog_response_cb), d);

	/* main table */
	t = gtk_table_new (3, 3, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(t), GUI_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(t), GUI_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER(t), GUI_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(w)->vbox), t, TRUE, TRUE, 0);

	/* bozo author */
	d->bozo_author_tb = tb = w = gtk_check_button_new_with_label (_("Add Author to Bozo Filter"));
	gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+1, atch, atch, 0, 0);
	w = gtk_entry_new ();
	gtk_widget_set_usize (w, wide_enough, -1);
	d->bozo_author_entry = w;
	gtk_entry_set_text (GTK_ENTRY(w), author);
	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
	++row;

	/* spam author */
	d->spam_author_tb = tb = w = gtk_check_button_new_with_label (_("Add Author to Spam Filter"));
	gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+1, atch, atch, 0, 0);
	w = gtk_entry_new ();
	gtk_widget_set_usize (w, wide_enough, -1);
	d->spam_author_entry = w;
	gtk_entry_set_text (GTK_ENTRY(w), author);
	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);
	++row;

	/* spam subject */
	d->spam_subject_tb = w = tb = gtk_check_button_new_with_label (_("Add Subject to Spam Filter"));
	gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+1, atch, atch, 0, 0);
	w = gtk_entry_new ();
	gtk_widget_set_usize (w, wide_enough, -1);
	d->spam_subject_entry = w;
	gtk_entry_set_text (GTK_ENTRY(w), subject);
	g_signal_connect (w, "changed", G_CALLBACK(activate_me_cb), tb);
	gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, atch, atch, 0, 0);

	/* cleanup */
	g_free (subject);

	return d->dialog;
}
